# Make sure that we are using the proper version of Distutils
# We make certain assumptions about the implementation
def EnsureVersion(version):
    """Checks Distutils version against specified version number"""
    # We need to use LooseVersions because of distutils in Python 2.1
    try:
        import distutils
    except ImportError:
        raise ImportError('Requires distutils v%s or newer. No distutils found.' %
            version)
    from distutils import __version__
    from distutils.version import LooseVersion
    dist_version = LooseVersion(__version__)
    expected = LooseVersion(version)
    if expected > dist_version:
        raise ImportError('Requires distutils v%s or newer.  Found version %s.' % (
            expected,
            dist_version,
            ))
    return
EnsureVersion('1.0')

def EnsureModules():
    """
    Make sure this Python installation has the modules we expect
    Python to have.
    """
    import sys
    sys_version = float(sys.version[:3])
    # module, min_version
    for name, version in [('pydoc', 2.1), ('inspect', 2.2)]:
        if sys_version < version:
            # use included modules
            compat_name = 'Ft.Lib.DistExt.Tools.' + name
            module = __import__(compat_name, {}, {}, '*')
            sys.modules[name] = module
    return
EnsureModules()

# For convienence of script writers
from distutils.extension import Extension
from Structures import *
