########################################################################
# $Header: /var/local/cvsroot/4Suite/Ft/Server/Client/Commands/Create/RawFile.py,v 1.13 2003/03/24 00:32:27 mbrown Exp $
"""
Implementation of '4ss create rawfile' command

Copyright 2003 Fourthought, Inc. (USA).
Detailed license and copyright information: http://4suite.org/COPYRIGHT
Project home, documentation, distributions: http://4suite.org/
"""

__doc__ = """Creates a new Raw File in the 4Suite repository.
This command is used to store non-XML based data in the repository.
If you are storing XML data, please see the 'create document' command."""

import sys
from Ft.Server.Client import FtServerClientException
from Ft.Lib import Uri
from Ft.Server.Client.Commands import CommandUtil

def Run(options,args):

    if args['sourceUri'] == '-':
        sourceStream = sys.stdin
    else:
        sourceStream = Uri.BASIC_RESOLVER.resolve(args['sourceUri'])

    text = sourceStream.read()

    name = args['name']
    aliases = args.get('alias',[])

    try:
        commit = 0
        repo = CommandUtil.GetRepository(options,'4ss.create.rawfile')
        try:
            doc = repo.createRawFile(name,options.get('imt','text/plain'),text)
            for alias in aliases:
                doc.addAlias(alias)

            commit = 1

        finally:
            if commit:
                repo.txCommit()
            else:
                repo.txRollback()

    except FtServerClientException, e:
        sys.stderr.write(str(e)+'\n')

    if args['sourceUri'] != '-':
        sourceStream.close()



def Register():

    from Ft.Lib.CommandLine import Options, Command, Arguments

    cmd = Command.Command('rawfile',
                          'Creates a new Raw File in the 4Suite repository',
                          '--imt=image/jpeg /images/4suite.jpg /local/path/to/4Suite.jpg',
                          __doc__,
                          function = Run,
                          arguments = [Arguments.RequiredArgument('name',
                                                                  "The name (repo path) of the new file",
                                                                  str),
                                       Arguments.RequiredArgument('sourceUri',
                                                                  "The URI to read the file's content from, or '-' for stdin",
                                                                  str),
                                       Arguments.ZeroOrMoreArgument('alias',
                                                                    "Additional aliases to assign to the new file.",
                                                                    str),
                                       ],
                          options = Options.Options([Options.Option(None,
                                                                    'imt=',
                                                                    'The file\'s Internet Media Type. Default: text/plain.'),
                                                     ]),
                          fileName = __file__,
                          )
    return cmd
