########################################################################
# $Header: /var/local/cvsroot/4Suite/Ft/Server/Client/Commands/Fetch/DocDef.py,v 1.8 2003/03/20 03:50:18 mbrown Exp $
"""
Implementation of '4ss fetch docdef' command

Copyright 2003 Fourthought, Inc. (USA).
Detailed license and copyright information: http://4suite.org/COPYRIGHT
Project home, documentation, distributions: http://4suite.org/
"""

__doc__ = """Fetches a Document Definition from the 4Suite repository.
For XSLT based DocDefs, fetches the XSLT document. For XPath based DocDefs, provides a formatted summary."""

import os, sys, time

from Ft.Server.Client import FtServerClientException
from Ft.Server.Client.Commands import CommandUtil
from Ft.Server.Common import ResourceTypes, ValidationInfo, AclConstants

def Run(options,args):
    try:
        repo = CommandUtil.GetRepository(options,'4ss.fetch.docdef')
        try:
            ddRoot = repo.getSystemDocDefContainer()
            dd = ddRoot.fetchResource(args['name'])

            print "Document Definition %s" % dd.getPath()
            print "-"*20
            print
            if dd.isResourceType(ResourceTypes.ResourceType.XSLT_DOCUMENT_DEFINITION):
                print "Transform for RDF generation:"
                print dd.getContent()
            else:
                print "Base Definitions: [%s]" % ','.join(map(lambda x:x.getUri(),dd.getBaseDefinitions()))
                print "Namespace mappings:"
                for name,value in dd.getNsMap().items():
                    print "%s --> %s" % (name,value)
                print
                print "Rdf Mappings:"
                for mapping in dd.getRdfMaps():
                    print "%s  %s  %s" % (mapping[0],mapping[1],mapping[2])
                print
                print "Read ACL: %s" % dd.getAcl(AclConstants.READ_ACCESS)
                print "Write ACL: %s" % dd.getAcl(AclConstants.WRITE_ACCESS)

                print
                cp = dd.getCreationParams()
                print "Full Text Indexing: %s" % cp.fullTextIndex
                if cp.extModules:
                    print "Xslt Extension Modules:"
                    for e in cp.extModules:
                        print "  %s" % e
                print "Enforce Schema: %s" % cp.enforceSchema

                vi = cp.validationInfo
                if vi.validationType != ValidationInfo.ValidationType.NONE:
                    raise "Finish"

        finally:
            repo.txRollback()
    except FtServerClientException, e:
        sys.stderr.write(str(e)+'\n')

    return


def Register():

    from Ft.Lib.CommandLine import Options, Command, Arguments

    cmd = Command.Command('docdef',
                          'Fetches a Document Definition from the 4Suite repository.',
                          'BASE_XML',
                          __doc__,
                          function = Run,
                          arguments = [Arguments.RequiredArgument('name',
                                                                  'Name (repo path) of document definition to view',
                                                                  str),
                                       ],

                          fileName = __file__,
                          )
    return cmd

