########################################################################
# $Header: /var/local/cvsroot/4Suite/Ft/Server/Client/Commands/Fetch/Group.py,v 1.7 2003/03/20 03:50:18 mbrown Exp $
"""
Implementation of '4ss fetch group' command

Copyright 2003 Fourthought, Inc. (USA).
Detailed license and copyright information: http://4suite.org/COPYRIGHT
Project home, documentation, distributions: http://4suite.org/
"""

__doc__ = """Fetches a Group from the 4Suite repository.
Lists the group's members and summarizes its ACL."""

import sys
from Ft.Server.Client import FtServerClientException
from Ft.Server.Client.Commands import CommandUtil
from Ft.Server.Common import AclConstants, ResourceTypes

def Run(options, args):

    try:
        name = args['groupName']
        repo = CommandUtil.GetRepository(options,'4ss.fetch.group')
        try:
            group = repo.fetchUserOrGroupByName(name)
            if group is not None:
                if group.isResourceType(ResourceTypes.ResourceType.GROUP):
                    print "Group %s:" % group.getName()
                    print "-"*10
                    print
                    print "Path: %s" % group.getPath()
                    print "Read ACL: %s" % group.getAcl(AclConstants.READ_ACCESS)
                    print "Write ACL: %s" % group.getAcl(AclConstants.WRITE_ACCESS)
                    print "Members:"
                    for m in group.getMembers():
                        print "  %s" % m.getName()
                else:
                    print "%s is a user, not a group." % name
            else:
                print "Group %s does not exist." % name
        finally:
            repo.txRollback()
    except FtServerClientException, e:
        sys.stderr.write(str(e)+'\n')

        return



def Register():

    from Ft.Lib.CommandLine import Options, Command, Arguments

    cmd = Command.Command('group',
                          "Lists information about a Group in the 4Suite repository.",
                          'developers',
                          __doc__,
                          function = Run,
                          arguments = [Arguments.RequiredArgument('groupName',
                                                                  'The name of the group to fetch',
                                                                  str),
                                       ],

                          fileName = __file__,
                          )
    return cmd

