########################################################################
#
# File Name:            Install.py
#
"""
WWW: http://4suite.org        e-mail: support@4suite.org

Copyright 2002 Fourthought Inc, USA.
See  http://4suite.org/COPYRIGHT  for license and copyright information
"""

__doc__ = """This command reads a 4Suite Application's setup XML file \
from the local filesystem and uses it to install, update or remove the \
application."""

import os, codecs
#import time, string
from Ft.Server.Client import FtServerClientException
from Ft.Server.Client.Commands import CommandUtil
from Ft import SHARE_DIR
from Ft.Server.Common.Install import InstallUtil
from Ft.Lib import Uri

def Run(options,args):

    baseDir = options.get('base','/')
    if options.has_key('demo'):
        #Get a Demo file
        #Get the Demo Directory
        file = os.path.join(SHARE_DIR, 'Demos', 'setup.xml')
        if not options.has_key('base'):
            baseDir = '/ftss/'
    elif options.has_key('dashboard'):
        #Get a Console file
        file = os.path.join(SHARE_DIR, '..', 'Server', 'Server', 'Dashboard',
                            'setup.xml')
        if not options.has_key('base'):
            baseDir = '/ftss/'

    elif not args.has_key('setup'):
        raise "You must specify either --demo or a setup file"
    else:
        file = args['setup']

    quiet = options.get('quiet')

    pl = InstallUtil.Deserialize(open(file),Uri.OsPathToUri(file))

    repo = CommandUtil.GetRepository(options, '4ss.install')

    commit = 0
    try:
        pl.install(repo,baseDir,quiet,checkAcl = not(options.has_key("no-acl")))
        commit = 1
    finally:
        try:
            if commit:
                repo.txCommit()
            else:
                repo.txRollback()
        except:
            pass


def GetAvailableProducts(filename):
    print "GetAvailableProducts:", filename


def Register():
    from Ft.Lib.CommandLine import Options, Command, Arguments
    cmd = Command.Command('install',
                          'Install/Update/Remove a 4Suite Application',
                          'GuestBook',
                          __doc__,
                          function = Run,
                          options=Options.Options([Options.Option(None,
                                                                  'demo',
                                                                  'Install the prepackaged demo'),
                                                   Options.Option(None,
                                                                  'dashboard',
                                                                  'Install the system dashboard'),
                                                   Options.Option('D',
                                                                  'delete',
                                                                  'delete an installed product'),
                                                   Options.Option('b',
                                                                  'base=',
                                                                  'base URI of the product in the repo'),
                                                   Options.Option('q',
                                                                  'quiet',
                                                                  "Don't display what is happening"),
                                                   Options.Option(None,
                                                                  'no-acl',
                                                                  "Don't sync ACL (install faster)"),
                                           ]),

                          arguments = [Arguments.OptionalArgument('setup',
                                                                  'The filename of the setup XML file.  If not specified, then you must specify the --demo flag.',
                                                                  str),
                                       ],
                          fileName = __file__,
                          )
    return cmd
