########################################################################
# $Header: /var/local/cvsroot/4Suite/Ft/Server/Client/Commands/Rdf/Add.py,v 1.10 2003/03/20 03:50:19 mbrown Exp $
"""
Implementation of '4ss rdf add' command

Copyright 2003 Fourthought, Inc. (USA).
Detailed license and copyright information: http://4suite.org/COPYRIGHT
Project home, documentation, distributions: http://4suite.org/
"""

__doc__ = """Adds statements to the RDF model.
A single statement can be given via command-line arguments, or multiple statements can be given in an RDF XML file that will be deserialized."""

import sys
from Ft.Lib import Uri
from Ft.Rdf.Statement import Statement
from Ft.Server.Client import FtServerClientException
from Ft.Server.Client.Commands import CommandUtil

def Run(options,args):
    try:
        commit = 0
        repo = CommandUtil.GetRepository(options, '4ss.rdf.add')
        try:
            scope = args['scope']
            doc = repo.fetchResource(scope)
            if options.get('document'):
                stream = Uri.BASIC_RESOLVER.resolve(options['document'])
                src = stream.read()
                doc.deserializeAndAdd(src)
            else:
                model = repo.getModel()
                s = Statement(args.get('subject'), args.get('predicate'),
                               args.get('object'), scope=scope
                               )
                model.add([s])
            commit = 1
        finally:
            try:
                if commit:
                    repo.txCommit()
                else:
                    repo.txRollback()
            except:
                pass
    except FtServerClientException, e:
        sys.stderr.write(str(e)+'\n')
        return


def Register():

    from Ft.Lib.CommandLine import Options, Command, Arguments

    cmd = Command.Command('add',
                          'Add RDF statements to a given scope',
                          '--document=spam.rdf /my/scope/path',
                          __doc__,
                          function = Run,
                          arguments = [Arguments.RequiredArgument('scope',
                                                                  "The scope into which the statements are to be added",
                                                                  str),
                                       Arguments.OptionalArgument('subject',
                                                                  "The subject of the new statement",
                                                                  str),
                                       Arguments.OptionalArgument('predicate',
                                                                  "The predicate of the new statement",
                                                                  str),
                                       Arguments.OptionalArgument('object',
                                                                  "The object of the new statement",
                                                                  str)],
                          options = Options.Options([Options.Option('d', 'document=', 'Add statements deserialized from the given document'),
                                      ]),
                          fileName = __file__,
                          )
    return cmd

