########################################################################
# $Header: /var/local/cvsroot/4Suite/Ft/Server/Client/Commands/Remove/Statement.py,v 1.6 2003/03/20 03:50:19 mbrown Exp $
"""
Implementation of '4ss remove statement' command

Copyright 2003 Fourthought, Inc. (USA).
Detailed license and copyright information: http://4suite.org/COPYRIGHT
Project home, documentation, distributions: http://4suite.org/
"""

__doc__ = """Removes a statement from the RDF model.
You must have write access to the statement."""

from Ft.Server.Client import FtServerClientException
from Ft.Server.Client.Commands import CommandUtil

from Ft.Rdf import Statement

def Run(options,args):

    try:
        commit = 0
        repo = CommandUtil.GetRepository(options,'4ss.remove.statement')
        try:
            model = repo.getModel()
            model.remove([Statement.Statement(args['subject'],
                                              args['predicate'],
                                              args['object'],
                                              args.get('uri',''),
                                              options.get('scope',''))])
            commit = 1
        finally:
            try:
                if commit:
                    repo.txCommit()
                else:
                    repo.txRollback()
            except:
                pass
    except FtServerClientException, e:
        raise


def Register():

    from Ft.Lib.CommandLine import Options, Command, Arguments

    cmd = Command.Command('statement',
                          """Removes a statement from the RDF model.""",
                          'ftss:///repo/path/to/doc.xml http://purl.org/dc/elements/1.1/title "Interesting Data"',
                          __doc__,
                          function = Run,
                          arguments = [Arguments.RequiredArgument('subject',
                                                                  "The subject of the statement to remove.",
                                                                  str),
                                       Arguments.RequiredArgument('predicate',
                                                                  "The predicate of the statement to remove.",
                                                                  str),
                                       Arguments.RequiredArgument('object',
                                                                  "The object of the statement to remove.",
                                                                  str),
                                       Arguments.OptionalArgument('uri',
                                                                  "The URI of the statement to remove.",
                                                                  str),
                                       ],

                          fileName = __file__,
                          )
    return cmd

