########################################################################
#
# File Name:            Render.py
#
"""
WWW: http://4suite.org        e-mail: support@4suite.org

Copyright (c) 2002 Fourthought Inc, USA.   All Rights Reserved.
See  http://4suite.org/COPYRIGHT  for license and copyright information
"""

import types, sys
from Ft.Server.Client import FtServerClientException
from Ft.Server.Client.Commands import CommandUtil
from Ft.Xml.Xslt._4xslt import g_paramBindingPattern

__doc__ = """This command will use 4xslt to render a source document \
with an XSLT stylesheet. Both documents must exist in a 4Suite repository. \
The metadata for a resource in the repository can be the source document."""


def Run(options, args):
    defs = options.get('define', [])
    if not isinstance(defs, types.ListType):
        defs = [defs]

    top_level_params = {}
    for d in defs:
        match = g_paramBindingPattern.match(d)
        top_level_params[match.group(1)] = match.group(2)

    output = sys.stdout
    quiet = options.get('quiet')
    try:
        repo = CommandUtil.GetRepository(options,'4ss.render')
        commit = 0
        try:
            doc = repo.fetchResource(args['path'])
            stys = []
            for path in args['stylesheet']:
                stys.append(doc.fetchResource(path))
            res, imt = doc.applyXslt(stys, params=top_level_params)
            if not quiet:
                sys.stderr.write("Internet Media Type: %s\n" % imt)
            output.write(res)
            commit = 1
        finally:
            try:
                if commit:
                    repo.txCommit()
                else:
                    repo.txRollback()
            except:
                pass
    except FtServerClientException, e:
        sys.stderr.write(str(e)+'\n')


def Register():

    from Ft.Lib.CommandLine import Options, Command, Arguments

    cmd = Command.Command('render',
                          """Apply XSLT against an XML document in the repository""",
                          '/fourthought.com',
                          __doc__,
                          function = Run,
                          arguments=[Arguments.RequiredArgument('path',
                                                                "The path of the source document in the repo.",
                                                                str),
                                     Arguments.OneOrMoreArgument('stylesheet',
                                                                 "The path of a stylesheet in the repo",
                                                                 str),
                                     ],
                          options=Options.Options([
                                     Options.Option('q', 'quiet', 'Present no output except for errors or any resulting document'),
                                     Options.Option('D',
                                                    'define=NAME=VALUE',
                                                    'Bind a top-level parameter'
                                                    ),
                          ]),
                          fileName = __file__,
                          )
    return cmd


