########################################################################
# $Header: /var/local/cvsroot/4Suite/Ft/Server/Client/Commands/Set/UserData.py,v 1.5 2003/03/20 03:50:20 mbrown Exp $
"""
Implementation of '4ss set userdata' command

Copyright 2003 Fourthought, Inc. (USA).
Detailed license and copyright information: http://4suite.org/COPYRIGHT
Project home, documentation, distributions: http://4suite.org/
"""

__doc__ = """Modifies the user data for a User in the 4Suite repository.
Can be used to set or replace one name-value pair at a time."""

from Ft.Server.Client import FtServerClientException
from Ft.Server.Client.Commands import CommandUtil
from Ft.Server.Common import ResourceTypes

def Run(options,args):

    try:
        name = args['userName']
        commit = 0
        repo = CommandUtil.GetRepository(options,'4ss.set.userdata')
        try:
            user = repo.fetchUserOrGroupByName(name)
            if user is not None and user.isResourceType(ResourceTypes.ResourceType.USER):
                if args.has_key('value'):
                    user[args['key']] = args['value']
                else:
                    del user[args['key']]
                commit = 1
            else:
                print "User %s does not exist." % name
        finally:
            try:
                if commit:
                    repo.txCommit()
                else:
                    repo.txRollback()
            except:
                pass
    except FtServerClientException, e:
        raise


def Register():

    from Ft.Lib.CommandLine import Options, Command, Arguments

    cmd = Command.Command('userdata',
                          'Modifies the user data for a User in the 4Suite repository.',
                          'michael age 26',
                          __doc__,
                          function = Run,
                          arguments = [Arguments.RequiredArgument('userName',
                                                                  "the name of the user on which to set data",
                                                                  str),
                                       Arguments.RequiredArgument('key',
                                                                  "the key (name) for the name-value pair",
                                                                  str),
                                       Arguments.OptionalArgument('value',
                                                                  "the value for the name-value pair; no value means delete the key",
                                                                  str),
                                       ],
                          fileName = __file__,
                          )
    return cmd


