<?xml version="1.0"?>
<!--
Useful FO links:

http://www.ibiblio.org/xml/books/bible2/chapters/ch18.html
http://www.xml.com/pub/a/2001/01/17/xsl-fo/
http://www.xml.com/pub/a/2001/01/24/xsl-fo/index.html
http://www.renderx.com/Tests/doc/html/tutorial.html
http://zvon.org/xxl/xslfoReference/Output/index.html

-->

<xsl:stylesheet
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:doc="http://docbook.org/docbook/xml/4.0/namespace"
  xmlns:fo="http://www.w3.org/1999/XSL/Format"
  xmlns:exslt="http://exslt.org/common"
  xmlns:f="http://xmlns.4suite.org/ext"
  extension-element-prefixes="f"
  version="1.0"
>

  <xsl:variable name='nodes' select='//node()'/>
  <xsl:variable name='pt-rtf'>
        <sz1>1pt</sz1>
        <sz2>3pt</sz2>
        <sz3>4pt</sz3>
        <sz4>5pt</sz4>
        <sz5>7pt</sz5>
        <sz6>8pt</sz6>
        <sz7>9pt</sz7>
        <sz8>11pt</sz8>
        <sz9>12pt</sz9>
        <sz12>16pt</sz12>
        <szpage>450</szpage>
  </xsl:variable>
  <xsl:variable name='pt' select='exslt:node-set($pt-rtf)'/>

  <xsl:preserve-space elements="doc:screen"/>

  <xsl:template match='doc:mediaobject'>
    <fo:block>
      <fo:external-graphic src='file:{doc:imageobject/doc:imagedata/@fileref}' scaling="uniform">
        <xsl:if test="$form='notes'">
          <xsl:attribute name="width"><xsl:value-of select="number(doc:imageobject/doc:imagedata/@width) div 2"/>px</xsl:attribute>
          <xsl:attribute name="height"><xsl:value-of select="number(doc:imageobject/doc:imagedata/@depth) div 2"/>px</xsl:attribute>
        </xsl:if>
      </fo:external-graphic>
    </fo:block>
  </xsl:template>

  <xsl:template match='doc:ulink'>
    <xsl:if test="@linkend">
      <xsl:message>Warning: you probably meant to use the attribute "url" rather than "linkend", which is wrong and will soon no longer work.</xsl:message>
    </xsl:if>
    <fo:basic-link external-destination="{@url|@linkend}" text-decoration="underline" color="green">
      <xsl:apply-templates/>
    </fo:basic-link>
  </xsl:template>

  <xsl:template match="doc:screen">
   <fo:block font-family="monospace" color="darkblue" font-size="{$pt/sz6}" white-space-collapse="false">
     <xsl:apply-templates/>
   </fo:block>
  </xsl:template>

  <xsl:template match="doc:para">
    <fo:block space-before="6pt">
      <xsl:apply-templates/>
    </fo:block>
  </xsl:template>

  <xsl:template match="doc:section/doc:title">
    <fo:block font-size="12pt" font-family="sans-serif"
        font-weight="bold" color="black"
        space-before="5pt" space-after="5pt">
      <xsl:apply-templates/>
    </fo:block>
  </xsl:template>

  <xsl:template match="doc:section">
   <fo:block space-before="8pt">
     <xsl:apply-templates/>
   </fo:block>
  </xsl:template>

  <xsl:template match="doc:itemizedlist|doc:simplelist">
<!--
    <fo:list-block line-height="{$pt/sz12}" font-family="Arial, Helvetica, sans" color="red" font-size="{$pt/sz8}">
-->
    <fo:list-block line-height="{$pt/sz8}" font-size="{$pt/sz6}">
      <xsl:apply-templates/>
    </fo:list-block>
  </xsl:template>

  <xsl:template match="doc:listitem">
    <fo:list-item>
      <fo:list-item-label start-indent="{$pt/sz7}" end-indent="{$pt/sz7}">
        <fo:block>*</fo:block>
      </fo:list-item-label>
      <fo:list-item-body start-indent="{$pt/sz12}">
        <fo:block><xsl:apply-templates/></fo:block>
      </fo:list-item-body>
    </fo:list-item>
  </xsl:template>

<!--

  <xsl:template match="doc:itemizedlist">
    <fo:block><xsl:apply-templates/></fo:block>
  </xsl:template>

  <xsl:template match="doc:listitem">
    <fo:block line-height="20px">
      <fo:external-graphic src="file:blue_btn.gif" height="auto" width="auto" content-height="auto" content-width="auto"/>  <xsl:apply-templates/>
    </fo:block>
  </xsl:template>

-->

  <xsl:attribute-set name="table-border">
    <xsl:attribute name="border-before-color">blue</xsl:attribute>
    <xsl:attribute name="border-before-width">2pt</xsl:attribute>
    <xsl:attribute name="border-after-color">blue</xsl:attribute>
    <xsl:attribute name="border-after-width">2pt</xsl:attribute>
    <xsl:attribute name="border-start-color">blue</xsl:attribute>
    <xsl:attribute name="border-start-width">2pt</xsl:attribute>
    <xsl:attribute name="border-end-color">blue</xsl:attribute>
    <xsl:attribute name="border-end-width">2pt</xsl:attribute>
  </xsl:attribute-set>

  <xsl:template match='doc:table'>
    <!--
    <fo:table-and-caption caption-side="top">
      <fo:table-caption>
        <fo:block>
          <xsl:value-of select="doc:title"/>
        </fo:block>
      </fo:table-caption>
-->
      <xsl:variable name="cols" select="count(doc:tgroup/doc:tbody/doc:row[1]/doc:entry)"/>
      <fo:table xsl:use-attribute-sets="table-border">
        <xsl:for-each select="$nodes[position() &lt;= $cols]">
          <fo:table-column column-width='{number($pt/szpage) div $cols}pt'/>
        </xsl:for-each>
        <!--
        <xsl:if test="@frame!='none'">
          <xsl:attribute name='border'>1pt</xsl:attribute>
        </xsl:if>
-->
        <fo:table-body>
          <xsl:apply-templates select="doc:tgroup/doc:tbody"/>
        </fo:table-body>
      </fo:table>
    <!--/fo:table-and-caption-->
  </xsl:template>

  <xsl:template match='doc:row'>
    <fo:table-row>
      <xsl:apply-templates/>
    </fo:table-row>
  </xsl:template>

  <xsl:template match='doc:entry'>
    <fo:table-cell>
      <fo:block font-size="{$pt/sz5}">
        <xsl:apply-templates/>        
      </fo:block>
    </fo:table-cell>
  </xsl:template>

</xsl:stylesheet>
