<?xml version="1.0"?>
<xsl:stylesheet
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:proj="http://Fourthought.com/namespaces/ProjectPlan"
  xmlns:doc="http://docbook.org/docbook/xml/4.0/namespace"
  version="1.0"
>

  <xsl:include href='docbook_html1.xslt'/>
  <xsl:include href='phrases.xslt'/>

  <xsl:template match="/">
    <HTML>
    <HEAD>
      <TITLE><xsl:value-of select='/proj:Project/@name'/></TITLE>
    </HEAD>
    <BODY>
    <H1>Project: <xsl:value-of select='/proj:Project/@name'/></H1>
    <xsl:apply-templates/>
    </BODY>
    </HTML>
  </xsl:template>

  <xsl:template match="proj:Overview">
	<H2>Overview</H2>
	<p><xsl:apply-templates/></p>
  </xsl:template>

  <xsl:template match="proj:Definition">
	<li><b><xsl:value-of select='@name'/>: </b><xsl:apply-templates/></li>
  </xsl:template>

  <xsl:template match="proj:Requirements">
	<H2>Requirements</H2>
	<xsl:apply-templates/>
  </xsl:template>

  <xsl:template match="proj:BusinessRequirements">
	<H3>Business-Level Requirements</H3>
	<ol>
	<xsl:apply-templates/>
	</ol>
  </xsl:template>

  <xsl:template match="proj:Requirement">
	<li><a name='{./@id}'/><b><xsl:value-of select='proj:Name'/>: </b><xsl:value-of select='proj:Description'/></li>

  </xsl:template>

  <xsl:template match="proj:FunctionalRequirements">
	<H3>Functional Requirements</H3>
	<OL>
	<xsl:apply-templates/>
	</OL>
	<BR/>
	<B>Total Estimated Effort</B>:
	<xsl:value-of select='sum(proj:Requirement/@effort)'/>
	man-hours
  </xsl:template>

  <xsl:template match="proj:Requirement">
	<LI><A HREF='#{./@bus-req}'><B><xsl:value-of select='proj:Name'/>(<xsl:value-of select='@effort'/> man-hours): </B></A><xsl:value-of select='proj:Description'/></LI>

  </xsl:template>

  <xsl:template match="proj:TestPlans">
	<H2>Test Plans</H2>
	<xsl:apply-templates/>
  </xsl:template>


  <xsl:template match="proj:TestPlan">
	<H3>For Business Requirement: <A HREF='#{./@BusReq}'>
	<xsl:value-of select="/proj:Project/proj:Requirements/proj:BusinessRequirements/proj:BusinessRequirement[@id=current()/@bus-req]/proj:Name"/>
	</A>
        </H3>
	<xsl:apply-templates/>
  </xsl:template>

  <xsl:template match="proj:Resource">
	<H2>Project Resources</H2>
	<H3><xsl:apply-templates select="proj:Resource[./@role='ProjectLead']"/></H3>
	<H3><xsl:apply-templates select="proj:Resource[./@role='TechnicalLead']"/></H3>
	<xsl:for-each select="proj:Resource[not(./@role='TechnicalLead')][not(./@role='ProjectLead')]">
	  <H3><xsl:value-of select='@role'/><xsl:text>: </xsl:text><xsl:value-of select='@name'/></H3>
	</xsl:for-each>
  </xsl:template>

  <xsl:template match="proj:Resource">
	<xsl:value-of select='@role'/><xsl:text>: </xsl:text><xsl:value-of select='@name'/>
  </xsl:template>

  <xsl:template match='proj:ResourceMapping'>
  <TABLE BORDER='1'>
    <TR><TH></TH>
    <xsl:for-each select='/proj:Project/proj:ResourceS/proj:Resource[not(./@role="ProjectLead")]'>
      <TH><xsl:value-of select='@name'/></TH>
    </xsl:for-each>
    </TR>
    <xsl:for-each select='/proj:Project/proj:Requirements/proj:FunctionslRequirements/proj:Requirement'>
	<xsl:variable name='curReq'><xsl:value-of select='string(./@ID)'/></xsl:variable>
	<TR>
	  <TD><xsl:value-of select='proj:Name'/>
	  </TD>
          <xsl:for-each select='/proj:Project/proj:ResourceS/proj:Resource[not(./@role="ProjectLead")]'>
            <TD>
	        <xsl:variable name='curRes'><xsl:value-of select='./@ID'/></xsl:variable>
		<xsl:choose>
		  <xsl:when test='/proj:Project/proj:ResourceMapping/proj:ResourceMap[@resource=$curRes][@func-req=$curReq]'>
		    Yes
		  </xsl:when>
		  <xsl:otherwise>
		    No
		  </xsl:otherwise>
		</xsl:choose>
            </TD>
          </xsl:for-each>
	</TR>
   </xsl:for-each>
   </TABLE>

  </xsl:template>

</xsl:stylesheet>

