<?xml version="1.0"?>
<xsl:stylesheet
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0" 
  xmlns:tech="http://FourThought.com/namespaces/Technical"
  xmlns:doc="http://docbook.org/docbook/xml/4.0/namespace"
>

  <xsl:output method="html"/>

  <xsl:param name="top-level-html" select="1"/>

  <xsl:template match="/">
    <xsl:choose>
      <xsl:when test="$top-level-html">
        <HTML>
        <HEAD>
          <TITLE>Use Case Specification</TITLE>
        </HEAD>
        <BODY>
          <xsl:apply-templates/>
        </BODY>
        </HTML>
      </xsl:when>
      <xsl:otherwise>
        <xsl:apply-templates/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:attribute-set name="tech:top-table-style">
    <xsl:attribute name="width">85%</xsl:attribute>
    <xsl:attribute name="cellpadding">0%</xsl:attribute>
    <xsl:attribute name="cellspacing">0%</xsl:attribute>
  </xsl:attribute-set>

  <xsl:preserve-space elements='doc:pre'/>
  <xsl:include href='docbook_html1.xslt'/>

  <xsl:template match="tech:UseCaseSpec">
	<H1><DIV ALIGN="CENTER">Use Case Specification</DIV></H1>
        <xsl:apply-templates select="tech:Header |
                                     tech:System |
                                     tech:Objective |
                                     tech:Actors |
                                     tech:Preconditions |
                                     tech:EventFlow |
                                     tech:Postconditions"/>
  </xsl:template>

  <xsl:template match="tech:Header">
	<BR/>
    <xsl:value-of select="@date"/><BR/>
    <xsl:value-of select="@desc"/><BR/>
    <xsl:apply-templates/>
	<BR/>
	<HR/>
  </xsl:template>

  <xsl:template match="tech:System">
	<H3>System Description</H3>
	<xsl:apply-templates/>
  </xsl:template>

  <xsl:template match="tech:Objective">
	<BR/>
	<HR/>
	<H3>Objective: <I><xsl:value-of select="@name"/></I></H3>
	<xsl:apply-templates select="doc:*"/>
  </xsl:template>

  <xsl:template match="tech:Actors">
	<H3>Actor(s)</H3>
    <xsl:for-each select='Actor'><I><xsl:value-of select='.'/></I></xsl:for-each>
  </xsl:template>

  <xsl:template match="tech:Preconditions">
	<H3>Precondition(s)</H3>
    <OL START='1' TYPE='1'>
	  <xsl:apply-templates select="tech:Condition"/>
    </OL>
  </xsl:template>

  <xsl:template match="tech:Postconditions">
	<H3>Postconditon(s)</H3>
    <OL START='1' TYPE='1'>
	  <xsl:apply-templates select="tech:Condition"/>
    </OL>
  </xsl:template>

  <xsl:template match="tech:Condition">
	<LI><xsl:apply-templates/></LI>
  </xsl:template>

  <xsl:template match="tech:EventFlow">
	<H3>Flow of Events</H3>
    <OL START='1' TYPE='1'>
	<xsl:apply-templates select="tech:Step"/>
    </OL>
  </xsl:template>

  <xsl:template match="tech:Step">
    <LI><xsl:apply-templates/></LI>
  </xsl:template>

</xsl:stylesheet>
