########################################################################
#
# File Name:            __init__.py
#
# Documentation:        http://docs.4suite.org/4Path/__init__.py.html
#
"""
WWW: http://4suite.org/4XPath         e-mail: support@4suite.org

Copyright (c) 2000-2001 Fourthought Inc, USA.   All Rights Reserved.
See  http://4suite.org/COPYRIGHT  for license and copyright information
"""

NAMESPACE_NODE = 10000
FT_OLD_EXT_NAMESPACE = 'http://xmlns.4suite.org/xpath/extensions'
FT_EXT_NAMESPACE = 'http://xmlns.4suite.org/ext'

import types
from xml.dom import Node
from Ft.Lib import boolean
from Ft.Lib.number import nan as NaN
from Ft.Lib.number import inf as Inf

from Ft import FtException

g_xpathPrimitiveTypes = [
    types.StringType,
    types.UnicodeType,
    types.IntType,
    types.LongType,
    types.FloatType,
    types.ListType,
    boolean.BooleanType,
    ]

g_xpathRecognizedNodes = [
        Node.ELEMENT_NODE,
        Node.ATTRIBUTE_NODE,
        Node.TEXT_NODE,
        Node.DOCUMENT_NODE,
        Node.PROCESSING_INSTRUCTION_NODE,
        Node.COMMENT_NODE
        ]

class XPathException(FtException):
    def __init__(self, errorCode, messages, args):
        FtException.__init__(self, errorCode, messages, args)

class CompiletimeException(XPathException):
    INTERNAL = 1
    SYNTAX = 2
    PROCESSING = 3

    def __init__(self, errorCode, *args):
        XPathException.__init__(self, errorCode, MessageSource.COMPILETIME, args)
        return

class RuntimeException(XPathException):
    INTERNAL = 1
    NO_CONTEXT = 10
    UNDEFINED_VARIABLE = 100
    UNDEFINED_PREFIX = 101
    UNDEFINED_FUNCTION = 102

    WRONG_ARGUMENTS = 200
    ARGCOUNT_NONE = 201
    ARGCOUNT_ATLEAST = 202
    ARGCOUNT_EXACT = 203
    ARGCOUNT_ATMOST = 204
    
    def __init__(self, errorCode, *args):
        XPathException.__init__(self, errorCode, MessageSource.RUNTIME, args)
        return


import MessageSource

def Evaluate(expr, contextNode=None, context=None):
    import os
    if os.environ.has_key('EXTMODULES'):
        ext_modules = os.environ["EXTMODULES"].split(':')
    else:
        ext_modules = []

    if contextNode and context:
        con = context.clone()
        con.node = contextNode
    elif context:
        con = context
    elif contextNode:
        #contextNode should be a node, not a context obj,
        #but this is a common error.  Be forgiving?
        if isinstance(contextNode, Context.Context):
            con = contextNode
        else:
            con = Context.Context(contextNode, 0, 0, extModuleList=ext_modules)
    else:
        raise RuntimeException(RuntimeException.NO_CONTEXT_ERROR)

    if hasattr(expr, "evaluate"):
        retval = expr.evaluate(con)
    else:
        retval = parser.new().parse(expr).evaluate(con)
    return retval


def Compile(expr):
    if type(expr) not in [types.StringType, types.UnicodeType]:
        raise TypeError("Expected string, found %s" % type(expr))
    try:
        return parser.new().parse(expr)
    except SyntaxError, error:
        raise CompiletimeException(CompiletimeException.SYNTAX, 0,0,str(error))
    except:
        import traceback, cStringIO
        stream = cStringIO.StringIO()
        traceback.print_exc(None, stream)
        raise RuntimeException(RuntimeException.INTERNAL, stream.getvalue())


def CreateContext(contextNode):
    return Context.Context(contextNode, 0, 0)



#Allow access to the NormalizeNode function
from Util import NormalizeNode

import Context

try:
    import XPathParserc
    parser = XPathParserc
except ImportError:
    import XPathParser
    parser = XPathParser

