########################################################################
# $Header: /var/local/cvsroot/4Suite/Ft/Xml/Xslt/HtmlWriter.py,v 1.24 2003/02/26 06:36:12 mbrown Exp $
"""
Implements the HTML output writer for XSLT processor output

Copyright 2003 Fourthought, Inc. (USA).
Detailed license and copyright information: http://4suite.org/COPYRIGHT
Project home, documentation, distributions: http://4suite.org/
"""

from Ft.Xml import EMPTY_NAMESPACE
from Ft.Xml.Lib import HtmlPrinter, HtmlPrettyPrinter
from Ft.Xml.Lib import XmlPrinter, XmlPrettyPrinter
from Ft.Xml.Xslt import XmlWriter

class HtmlWriter(XmlWriter.XmlWriter):
    """
    Takes events such as those generated by an XSLT processor and
    emits proper HTML. Switches to XML output for elements in a
    non-empty namespace.
    """
    def __init__(self, *args, **kwargs):
        apply(XmlWriter.XmlWriter.__init__, (self,) + args, kwargs)
        self._htmlprinter = None
        self._xmlprinter = None
        self._xmlprettyprinter = None
        self._printerstack = [0] # 0=html, 1=xml, 2=xmlpretty

    def _completeLastElement(self):
        # Test now since XmlWriter clears self._element before returning
        head_tag = self._element and self._element.lower() == u'head'

        XmlWriter.XmlWriter._completeLastElement(self)

        if head_tag and not self._printerstack[-1]:
            content = u'%s; charset=%s' % (self._outputParams.mediaType,
                                           self._outputParams.encoding)
            attrs = {u'http-equiv' : u'Content-Type',
                     u'content' : content,
                     }
            self._printer.startElement(u'meta', {}, attrs)
            self._printer.endElement(u'meta')
        return

    def startDocument(self):
        self._outputParams.setDefault('encoding', 'iso-8859-1')
        self._outputParams.setDefault('indent', 1)
        self._outputParams.setDefault('mediaType', 'text/html')

        if self._outputParams.indent:
            printer = HtmlPrettyPrinter.HtmlPrettyPrinter
        else:
            printer = HtmlPrinter.HtmlPrinter
        self._htmlprinter = printer(self._stream,
                                self._outputParams.encoding.encode('ascii'))
        self._printer = self._htmlprinter
        self._printerstack.append(0)
        self._printer.startDocument()
        return

    def startElement(self, tagName, namespace=EMPTY_NAMESPACE, extraNss=None):
        if self._outputParams.indent:
            indent_level = self._printer._level
            indent_forbidden = self._printer._indentForbidden
            is_inline = self._printer._isInline
        if namespace != EMPTY_NAMESPACE:
            # elements in a non-empty namespace are written as XML
            if not self._xmlprinter:
                self._xmlprinter = XmlPrinter.XmlPrinter(self._stream,
                                    self._outputParams.encoding.encode('ascii'))
                self._xmlprettyprinter = XmlPrettyPrinter.XmlPrettyPrinter(
                                          self._stream,
                                          self._outputParams.encoding.encode('ascii'))
            if self._outputParams.indent and not self._printer._indentForbidden:
                self._printer = self._xmlprettyprinter
                self._printerstack.append(2)
            else:
                self._printer = self._xmlprinter
                self._printerstack.append(1)
        else:
            self._printer = self._htmlprinter
            self._printerstack.append(0)
        if self._outputParams.indent:
            self._printer._level = indent_level
            self._printer._indentForbidden = indent_forbidden
            self._printer._isInline = is_inline
        XmlWriter.XmlWriter.startElement(self, tagName, namespace, extraNss)
        return

    def endElement(self, tagName):
        XmlWriter.XmlWriter.endElement(self, tagName)
        if self._outputParams.indent:
            indent_level = self._printer._level
            indent_forbidden = self._printer._indentForbidden
        self._printerstack.pop()
        if self._printerstack[-1] == 2:
            self._printer = self._xmlprettyprinter
        elif self._printerstack[-1] == 1:
            self._printer = self._xmlprinter
        else:
            self._printer = self._htmlprinter
        if self._outputParams.indent:
            self._printer._level = indent_level
            self._printer._indentForbidden = indent_forbidden
        return


