#!/usr/bin/env python
########################################################################
# $Header: /var/local/cvsroot/4Suite/Ft/Xml/Xslt/_4xslt.py,v 1.34 2003/06/18 05:25:21 mbrown Exp $
"""
Command-line invocation of the 4XSLT processor

Copyright 2003 Fourthought, Inc. (USA).
Detailed license and copyright information: http://4suite.org/COPYRIGHT
Project home, documentation, distributions: http://4suite.org/
"""

import re, os, sys, traceback, cPickle, time
from Ft.Xml import XPath, InputSource
from Ft.Xml.Xslt import XsltException
from Ft.Xml.Xslt import Processor
from Ft.Xml import __version__
from Ft.Lib import Uri, UriException

__doc__ = """4XSLT command-line application"""

g_paramBindingPattern = re.compile(r"([\d\D_\.\-]*:?[\d\D_\.\-]+)=(.*)")

from Ft import MAX_PYTHON_RECURSION_DEPTH
sys.setrecursionlimit(MAX_PYTHON_RECURSION_DEPTH)

def Run(options,args):

    if options.has_key('version'):
        print '4XSLT, from 4Suite %s' % __version__
        return

    if not args.has_key('source-uri'):
        print 'A source URI argument is required. See "4xslt -h" for usage info.'
        return

    include_paths = options.get('includepath', [])
    if type(include_paths) != type([]):
        include_paths = [include_paths]

    xinclude = not options.has_key('noxinclude')
    ignore_pis = options.has_key('ignore')
    if options.has_key('compile'):
        output_flags = "wb"
    else:
        output_flags = "w"
    out_file = options.has_key('outfile') \
               and open(options['outfile'], output_flags) \
               or sys.stdout
    validate_flag = options.has_key('validate')
    checktime = options.has_key('time')

    # Do the file checking here; the rest of 4XSLT is uri based
    source = args['source-uri']
    if os.path.isfile(source):
        #source = urllib.pathname2url(source)
        source = Uri.OsPathToUri(source, attemptAbsolute=1)

    stylesheets = []
    for uri in args['stylesheet-uri']:
        if os.path.isfile(uri):
            uri = Uri.OsPathToUri(uri, attemptAbsolute=1)
            #uri = urllib.pathname2url(uri)
        stylesheets.append(uri)

    if options.has_key('compile') or options.has_key('reflex'):
        stylesheets = [source]

    stacktrace_on_error = options.has_key('stacktrace-on-error')

    if options.has_key('trace') or options.has_key('debug'):
        if not options.has_key('outfile'):
            raise SystemExit("If you are using the trace, debug, or profile options, you must specify an output file for the XSLT transform results")

        from Ft.Xml.Xslt.Debugger import DebugController
        from Ft.Xml.Xslt import ExtendedProcessingElements, StylesheetHandler
        processor = ExtendedProcessingElements.ExtendedProcessor(stylesheetIncPaths=include_paths)
        StylesheetHandler._ELEMENT_MAPPING = ExtendedProcessingElements.GetMappings()
        if options.has_key('trace'):
            processor._4xslt_trace = 1
            if options.has_key('trace-file'):
                processor._4xslt_traceStream = open(options['trace-file'],'w')
        if options.has_key('debug'):
            processor._4xslt_debug = 1
            processor._ft_debug_controller = DebugController.DebugController()
    else:
        processor = Processor.Processor(stylesheetIncPaths=include_paths)

    #Attempt to register EXT modules
    moduleList = os.environ.get("EXTMODULES")
    if moduleList:
        processor.registerExtensionModules(moduleList.split(":"))

    #Apply top-level parameter overrides
    defs = options.get('define', [])
    if type(defs) != type([]):
        defs = [defs]

    top_level_params = {}
    for d in defs:
        match = g_paramBindingPattern.match(d)
        top_level_params[match.group(1)] = match.group(2)

    from Ft.Xml import Domlette

    if validate_flag:
        reader = Domlette.ValidatingReader
    else:
        reader = Domlette.NonvalidatingReader

    try:
        processor.setDocumentReader(reader)
        for sty in stylesheets:
            isrc = None
            try:
                isrc = InputSource.DefaultFactory.fromUri(sty)
            except (OSError, UriException), error:
                for path in processor._stylesheetIncPaths:
                    try:
                        new_sty = Uri.BASIC_RESOLVER.normalize(sty, path)
                        isrc = InputSource.DefaultFactory.fromUri(new_sty)
                        break
                    except (OSError, UriException):
                        pass
                if not isrc:
                    raise error
            except:
                traceback.print_exc(1000, sys.stderr)
            if options.has_key('instant'):
                istyle = cPickle.load(isrc)
                processor.appendStylesheetInstance(istyle.stylesheet,
                                                   refUri=isrc.uri)
            else:
                processor.appendStylesheet(isrc)

        if options.has_key('compile'):
            cPickle.dump(processor.stylesheet.root, out_file, 1)
            return

        if source == '-':
            isrc = InputSource.InputSource(sys.stdin,'.')
        else:
            isrc = InputSource.DefaultFactory.fromUri(source)

	if checktime:
            start = time.time()
        processor.run(isrc, ignore_pis, topLevelParams=top_level_params,
                      outputStream=out_file)
        if checktime:
            end = time.time()
    except XsltException, error:
        if stacktrace_on_error:
            traceback.print_exc(1000, sys.stderr)
        raise SystemExit(''.join([c.encode(sys.getdefaultencoding(), 'ignore')
                                  or "&#%d;" % ord(c) for c in error.message]))
    except (XPath.RuntimeException, XPath.CompiletimeException), error:
        if stacktrace_on_error:
            traceback.print_exc(1000, sys.stderr)
        raise SystemExit(''.join([c.encode(sys.getdefaultencoding(), 'ignore')
                                  or "&#%d;" % ord(c) for c in error.message]))
    except Exception, error:
        if stacktrace_on_error:
            traceback.print_exc(1000, sys.stderr)
            msg = ''
        else:
            msg = 'An unexpected error occurred while processing.\n' + \
                  'The error was: %s: %s\n' % (str(sys.exc_type)[11:], str(error)) + \
                  'Use the -e (stacktrace-on-error) option for a full stack trace.'
        raise SystemExit(''.join([c.encode(sys.getdefaultencoding(), 'ignore')
                                  or "&#%d;" % ord(c) for c in msg]))

    if out_file.isatty():
        out_file.flush()
        sys.stderr.write('\n')
        if checktime:
            print "Transformation time: %dms" % (1000 * (end - start))
    out_file.close()
    return


from Ft.Lib.CommandLine import Options, CommandLineApp, Arguments, Command


class XsltCommandLineApp(CommandLineApp.CommandLineApp):
    def __init__(self):
        CommandLineApp.CommandLineApp.__init__(
            self,
            '4xslt',
            '4XSLT version %s' % __version__,
            __doc__,
            [],
            ourOptions = Options.Options([Options.Option('V',
                                                         'version',
                                                         'Display program version and exit',
                                                         ),
                                          Options.Option('v',
                                                         'validate',
                                                         'Validate the input file as it is being parsed',
                                                         ),
                                          Options.Option('i',
                                                         'ignore',
                                                         'Ignore <?xml-stylesheet ...?> instructions'
                                                         ),
                                          Options.Option('D',
                                                         'define=NAME=VALUE',
                                                         'Bind a top-level parameter'
                                                         ),
                                          Options.Option('I',
                                                         'includepath=URI',
                                                         'Set the base URI for imports and includes'
                                                         ),
                                          Options.Option('o',
                                                         'outfile=FILE',
                                                         'Direct transformation output to FILE '\
                                                         '(file will be overwritten if it exists)'
                                                         ),
                                          Options.Option('e',
                                                         'stacktrace-on-error',
                                                         'Display a stack trace when an error occurs',
                                                         ),
                                          Options.Option(None,
                                                         'noxinclude',
                                                         'Do not expand XIncludes in source or stylesheet'
                                                         ),
                                          Options.Option(None,
                                                         'trace',
                                                         'Send execution trace output to stdout or file',
                                                         ),
                                          Options.Option(None,
                                                         'trace-file=FILE',
                                                         'Set trace file for execution trace output',
                                                         ),
                                          Options.Option(None,
                                                         'reflex',
                                                         'The stylesheet is also the source document.  Useful for cases where the stylesheet doesn\'t really use the source.'
                                                         ),
                                          Options.Option(None,
                                                         'compile',
                                                         'Compile an instant stylesheet.  The result is written to stdout, unless -o is used.'
                                                         ),
                                          Options.Option(None,
                                                         'instant',
                                                         'The stylesheet passed in is an "instant" (compiled) stylesheet.  Only one stylesheet can be specified with this option.'
                                                         ),
                                          Options.Option(None,
                                                         'time',
                                                         'Display the elapsed transformation time on stderr.'
                                                         ),
                                          #Options.Option('d',
                                          #               'debug',
                                          #               'Execute 4xslt in the command line debugger',
                                          #               ),
                                          ]),
            enableShowCommands = 0
            )

        self.function = Run
        self.arguments = [Arguments.OptionalArgument(
            'source-uri',
            'The URI of the source XML document. Required unless running "4xslt -V",'\
            'or with --reflex or --compile.'\
            ' Note: if you use "-" as the name of the source document, the source will'\
            ' instead be read from standard input.',
            str),
                          Arguments.ZeroOrMoreArgument('stylesheet-uri',
                                                      'The URI(s) of the stylesheets to apply to the source XML.',
                                                      str),
                          ]

    def validate_arguments(self,args):
        return Command.Command.validate_arguments(self,args)


def Register():
    return XsltCommandLineApp()

